@@ -4,6 +4,8 @@ import os
 import logging
 import stat
 import shutil
+import gettext
+from pathlib import Path
 
 from gi.repository import GLib, Gio, GObject
 
@@ -12,6 +14,8 @@ from util import FileType
 import prefs
 import warp_pb2
 
+_ = gettext.gettext
+
 FILE_INFOS = ",".join([
     "standard::size",
     "standard::allocated-size",
@@ -164,6 +168,7 @@ class FileReceiver(GObject.Object):
     def __init__(self, op):
         super(FileReceiver, self).__init__()
         self.save_path = prefs.get_save_path()
+        self.save_path_obj = Path(self.save_path).resolve()
         self.op = op
         self.preserve_perms = prefs.preserve_permissions() and util.save_folder_is_native_fs()
         self.preserve_timestamp = prefs.preserve_timestamp() and util.save_folder_is_native_fs()
@@ -176,18 +181,20 @@ class FileReceiver(GObject.Object):
         self.current_mtime = 0
         self.current_mtime_usec = 0
 
-        if op.existing:
-            for name in op.top_dir_basenames:
-                try:
-                    path = os.path.join(self.save_path, name)
-                    if os.path.isdir(path): # file not found is ok
-                        shutil.rmtree(path)
-                    else:
-                        os.remove(path)
-                except FileNotFoundError:
-                    pass
-                except Exception as e:
-                    logging.warning("Problem removing existing files.  Transfer may not succeed: %s" % e)
+        self.remaining_files = op.total_count
+        self.remaining_bytes = op.total_size
+
+        for name in op.top_dir_basenames:
+            try:
+                path = os.path.join(self.save_path, name)
+                if os.path.isdir(path): # file not found is ok
+                    shutil.rmtree(path)
+                else:
+                    os.remove(path)
+            except FileNotFoundError:
+                pass
+            except Exception as e:
+                logging.warning("Problem removing existing files.  Transfer may not succeed: %s" % e)
 
         # We write files top-down.  If we're preserving permissions and we receive
         # a folder in some hierarchy that is not writable, we won't be able to create
@@ -205,8 +212,17 @@ class FileReceiver(GObject.Object):
             self.current_type = s.file_type
             self.current_mtime = s.time.mtime
             self.current_mtime_usec = s.time.mtime_usec
+        if self.remaining_files == 0:
+            raise Exception(_("File count exceeds original request size"))
 
         if not self.current_gfile:
+            # Check for valid path (pathlib.Path resolves both relative and symbolically-linked paths)
+            test_path = Path(path).resolve()
+            try:
+                test_path.relative_to(self.save_path_obj)
+            except ValueError:
+                raise ReceiveError(_("Resolved path is not valid: %s -> %s") % (path, str(test_path)), fatel=True)
+
             self.current_gfile = Gio.File.new_for_path(path)
 
         if s.file_type == FileType.DIRECTORY:
@@ -215,8 +231,7 @@ class FileReceiver(GObject.Object):
             make_symbolic_link(self.op, path, s.symlink_target)
         else:
             if self.current_stream == None:
-                flags = Gio.FileCreateFlags.REPLACE_DESTINATION
-                self.current_stream = self.current_gfile.replace(None, False, flags, None)
+                self.current_stream = self.current_gfile.create(Gio.FileCreateFlags.NONE, None)
 
             if not s.chunk:
                 return
@@ -265,6 +280,7 @@ class FileReceiver(GObject.Object):
         self.current_mode = 0
         self.current_path = None
         self.current_gfile = None
+        self.remaining_files -= 1
 
     def apply_folder_permissions(self):
         if self.preserve_perms:
